"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const DesignType_1 = require("./DesignType");
const qrcode_1 = __importDefault(require("qrcode"));
const BYTE_LENGTH_NORMAL = 620;
const BYTE_LENGTH_PRO = 540;
class QRData {
    constructor(type, index, parity, bytes) {
        // Make sure we have valid parameters
        if (type === DesignType_1.DesignType.Normal) {
            if (index !== undefined || parity !== undefined) {
                throw new Error('Index and parity must be set to "undefined" for normal designs');
            }
        }
        else if (type === DesignType_1.DesignType.Pro) {
            if (index === undefined || index < 0 || index > 3) {
                throw new Error('Must specify a QR code index between 0-3 for pro designs');
            }
        }
        else {
            throw new Error('Invalid design type');
        }
        this.type = type;
        this.index = index;
        this.parity = parity;
        this.bytes = bytes || new Uint8Array(this.byteLength());
    }
    static fromBytes(bytes, index, parity) {
        if (bytes.length === BYTE_LENGTH_NORMAL) {
            return new QRData(DesignType_1.DesignType.Normal, index, parity, bytes);
        }
        else if (bytes.length === BYTE_LENGTH_PRO) {
            return new QRData(DesignType_1.DesignType.Pro, index, parity, bytes);
        }
        throw new Error('Invalid data length');
    }
    byteLength() {
        return (this.type === DesignType_1.DesignType.Pro) ? BYTE_LENGTH_PRO : BYTE_LENGTH_NORMAL;
    }
    /**
     * The underlying bytes for this QR data segment.
     */
    get bytes() { return this._bytes; }
    set bytes(value) {
        if (value.length !== this.byteLength()) {
            throw new Error('Invalid data length');
        }
        this._bytes = value;
    }
    // QR code conversion
    qrSegments() {
        const segments = [{ mode: 'byte', data: this.bytes }];
        if (this.type === DesignType_1.DesignType.Pro) {
            if (this.index === undefined) {
                throw new Error('Index is required');
            }
            if (this.parity === undefined) {
                throw new Error('Parity byte is required');
            }
            segments.unshift({ mode: 'structuredappend', data: { position: this.index, total: 3, parity: this.parity } });
        }
        return segments;
    }
    toFile(path) {
        return qrcode_1.default.toFile(path, this.qrSegments());
    }
    toFileStream(stream) {
        // node-qrcode type data is incorrect, QRCode.toFileStream does not actually return a promise here
        return new Promise((resolve, reject) => {
            stream.on('close', resolve);
            stream.on('error', reject);
            qrcode_1.default.toFileStream(stream, this.qrSegments());
        });
    }
    toDataUrl(type = 'png') {
        return qrcode_1.default.toDataURL(this.qrSegments(), { type: `image/${type}` });
    }
    toCanvas(canvas) {
        return qrcode_1.default.toCanvas(canvas, this.qrSegments());
    }
}
exports.QRData = QRData;
